@@ -1,39 +1,52 @@
 package gnu.crypto.cipher;
 
 // ----------------------------------------------------------------------------
-// $Id: Khazad.java,v 1.6 2002/06/28 13:14:28 raif Exp $
+// $Id: Khazad.java,v 1.9 2003/04/28 10:29:42 raif Exp $
 //
-// Copyright (C) 2001-2002, Free Software Foundation, Inc.
+// Copyright (C) 2001, 2002, 2003, Free Software Foundation, Inc.
 //
-// This program is free software; you can redistribute it and/or modify it
-// under the terms of the GNU General Public License as published by the Free
-// Software Foundation; either version 2 of the License or (at your option) any
-// later version.
+// This file is part of GNU Crypto.
 //
-// This program is distributed in the hope that it will be useful, but WITHOUT
-// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-// more details.
+// GNU Crypto is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; either version 2, or (at your option)
+// any later version.
 //
-// You should have received a copy of the GNU General Public License along with
-// this program; see the file COPYING.  If not, write to the
+// GNU Crypto is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+// General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; see the file COPYING.  If not, write to the
 //
 //    Free Software Foundation Inc.,
 //    59 Temple Place - Suite 330,
 //    Boston, MA 02111-1307
 //    USA
 //
-// As a special exception, if you link this library with other files to produce
-// an executable, this library does not by itself cause the resulting
-// executable to be covered by the GNU General Public License.  This exception
-// does not however invalidate any other reasons why the executable file might
-// be covered by the GNU General Public License.
+// Linking this library statically or dynamically with other modules is
+// making a combined work based on this library.  Thus, the terms and
+// conditions of the GNU General Public License cover the whole
+// combination.
+//
+// As a special exception, the copyright holders of this library give
+// you permission to link this library with independent modules to
+// produce an executable, regardless of the license terms of these
+// independent modules, and to copy and distribute the resulting
+// executable under terms of your choice, provided that you also meet,
+// for each linked independent module, the terms and conditions of the
+// license of that module.  An independent module is a module which is
+// not derived from or based on this library.  If you modify this
+// library, you may extend this exception to your version of the
+// library, but you are not obligated to do so.  If you do not wish to
+// do so, delete this exception statement from your version.
 // ----------------------------------------------------------------------------
 
 import gnu.crypto.Registry;
 import gnu.crypto.util.Util;
 
-import java.io.PrintWriter;
+//import java.io.PrintWriter;
 import java.security.InvalidKeyException;
 import java.util.ArrayList;
 import java.util.Collections;
@@ -51,24 +64,24 @@ import java.util.Iterator;
  * <ol>
  *    <li><a href="http://planeta.terra.com.br/informatica/paulobarreto/KhazadPage.html">The
  *    Khazad Block Cipher</a>.<br>
- *    <a href="mailto:pbarreto@scopus.com.br">Paulo S.L.M. Barreto</a> and
+ *    <a href="mailto:paulo.barreto@terra.com.br">Paulo S.L.M. Barreto</a> and
  *    <a href="mailto:vincent.rijmen@esat.kuleuven.ac.be">Vincent Rijmen</a>.</li>
  * </ol>
  *
- * @version $Revision: 1.6 $
+ * @version $Revision: 1.9 $
  */
 public final class Khazad extends BaseCipher {
 
    // Debugging methods and variables
    // -------------------------------------------------------------------------
 
-   private static final String NAME = "khazad";
+//   private static final String NAME = "khazad";
    private static final boolean DEBUG = false;
    private static final int debuglevel = 9;
-   private static final PrintWriter err = new PrintWriter(System.out, true);
-   private static void debug(String s) {
-      err.println(">>> "+NAME+": "+s);
-   }
+//   private static final PrintWriter err = new PrintWriter(System.out, true);
+//   private static void debug(String s) {
+//      err.println(">>> "+NAME+": "+s);
+//   }
 
    // Constants and variables
    // -------------------------------------------------------------------------
