@@ -1,9 +1,9 @@
 package gnu.crypto.mac;
 
 // ----------------------------------------------------------------------------
-// $Id: UHash32.java,v 1.5 2002/11/07 17:17:45 raif Exp $
+// $Id: UHash32.java,v 1.6 2003/04/28 10:50:38 raif Exp $
 //
-// Copyright (C) 2001, 2002, Free Software Foundation, Inc.
+// Copyright (C) 2001, 2002, 2003 Free Software Foundation, Inc.
 //
 // This file is part of GNU Crypto.
 //
@@ -45,11 +45,10 @@ package gnu.crypto.mac;
 
 import gnu.crypto.cipher.IBlockCipher;
 import gnu.crypto.prng.IRandom;
-import gnu.crypto.prng.UMacGenerator;
 import gnu.crypto.prng.LimitReachedException;
+import gnu.crypto.prng.UMacGenerator;
 
 import java.io.ByteArrayOutputStream;
-import java.io.IOException;
 import java.math.BigInteger;
 import java.security.InvalidKeyException;
 import java.util.HashMap;
@@ -89,7 +88,7 @@ import java.util.Map;
  *    T. Krovetz, J. Black, S. Halevi, A. Hevia, H. Krawczyk, and P. Rogaway.</li>
  * </ol>
  *
- * @version $Revision: 1.5 $
+ * @version $Revision: 1.6 $
  */
 public class UHash32 extends BaseMac {
 
@@ -113,15 +112,15 @@ public class UHash32 extends BaseMac {
             (byte) 0xFF, (byte) 0xFF, (byte) 0xFF, (byte) 0x61
          });
 
-   private static final BigInteger TWO = BigInteger.valueOf(2L);
-   private static final long BOUNDARY = TWO.shiftLeft(17).longValue();
+   static final BigInteger TWO = BigInteger.valueOf(2L);
+   static final long BOUNDARY = TWO.shiftLeft(17).longValue();
 
    // 2**64 - 2**32
-   private static final BigInteger LOWER_RANGE = TWO.pow(64).subtract(TWO.pow(32));
+   static final BigInteger LOWER_RANGE = TWO.pow(64).subtract(TWO.pow(32));
    // 2**128 - 2**96
-   private static final BigInteger UPPER_RANGE = TWO.pow(128).subtract(TWO.pow(96));
+   static final BigInteger UPPER_RANGE = TWO.pow(128).subtract(TWO.pow(96));
 
-   private static final byte[] ALL_ZEROES = new byte[32];
+   static final byte[] ALL_ZEROES = new byte[32];
 
    int streams;
    L1Hash32[] l1hash;
@@ -144,8 +143,10 @@ public class UHash32 extends BaseMac {
 
       this.streams = that.streams;
       if (that.l1hash != null) {
-         this.l1hash = new L1Hash32[that.l1hash.length];
-         for (int i = 0; i < that.l1hash.length; i++) {
+//         this.l1hash = new L1Hash32[that.l1hash.length];
+         this.l1hash = new L1Hash32[that.streams];
+//         for (int i = 0; i < that.l1hash.length; i++) {
+         for (int i = 0; i < that.streams; i++) {
             if (that.l1hash[i] != null) {
                this.l1hash[i] = (L1Hash32) that.l1hash[i].clone();
             }
@@ -173,15 +174,14 @@ public class UHash32 extends BaseMac {
     * @param n a number of bits.
     * @return the largest prime number less than 2**n.
     */
-   private static final BigInteger prime(int n) {
+   static final BigInteger prime(int n) {
       switch (n) {
       case 19: return PRIME_19;
       case 32: return PRIME_32;
       case 36: return PRIME_36;
       case 64: return PRIME_64;
       case 128: return PRIME_128;
-      default: throw new IllegalArgumentException(
-            "Undefined prime("+String.valueOf(n)+")");
+      default: throw new IllegalArgumentException("Undefined prime("+String.valueOf(n)+")");
       }
    }
 
@@ -191,7 +191,7 @@ public class UHash32 extends BaseMac {
    // java.lang.Cloneable interface implementation ----------------------------
 
    public Object clone() {
-      return (new UHash32(this));
+      return new UHash32(this);
    }
 
    // gnu.crypto.mac.IMac interface implementation ----------------------------
@@ -205,9 +205,9 @@ public class UHash32 extends BaseMac {
       byte[] K = (byte[]) attributes.get(MAC_KEY_MATERIAL);
       if (K == null) {
          throw new InvalidKeyException("Null Key");
-      } else if (K.length != UMac32.KEY_LEN) {
-         throw new InvalidKeyException("Invalid Key length: "
-               +String.valueOf(K.length));
+      }
+      if (K.length != UMac32.KEY_LEN) {
+         throw new InvalidKeyException("Invalid Key length: "+String.valueOf(K.length));
       }
 
       // Calculate iterations needed to make UMAC-OUTPUT-LEN bytes
@@ -287,14 +287,11 @@ public class UHash32 extends BaseMac {
 
    public byte[] digest() {
       byte[] result = new byte[UMac32.OUTPUT_LEN];
-
       for (int i = 0; i < streams; i++) {
          byte[] partialResult = l1hash[i].digest();
          System.arraycopy(partialResult, 0, result, 4*i, 4);
       }
-
       reset();
-
       return result;
    }
 
@@ -325,7 +322,7 @@ public class UHash32 extends BaseMac {
       private byte[] buffer; // work buffer L1_KEY_LEN long
       private int count; // meaningful bytes in buffer
       private ByteArrayOutputStream Y;
-      private byte[] y;
+//      private byte[] y;
       private long totalCount;
       private L2Hash32 l2hash;
       private L3Hash32 l3hash;
@@ -400,10 +397,11 @@ public class UHash32 extends BaseMac {
 
          // For each chunk, except the last: endian-adjust, NH hash
          // and add bit-length.  Use results to build Y.
+         buffer[count] = b;
          count++;
          totalCount++;
          if (count >= UMac32.L1_KEY_LEN) {
-            y = nh32(UMac32.L1_KEY_LEN);
+            byte[] y = nh32(UMac32.L1_KEY_LEN);
             Y.write(y, 0, 8);
 
             count = 0;
@@ -412,8 +410,8 @@ public class UHash32 extends BaseMac {
             // If length(M) <= L1_KEY_LEN, then skip L2-HASH.
             if (Y.size() == 16) { // we already hashed twice L1_KEY_LEN
                byte[] A = Y.toByteArray();
-               l2hash.update(A, 0, 16);
                Y.reset();
+               l2hash.update(A, 0, 16);
             }
          }
       }
@@ -427,11 +425,12 @@ public class UHash32 extends BaseMac {
                System.arraycopy(ALL_ZEROES, 0, buffer, count, limit - count);
                count += limit - count;
             }
-            y = nh32(count);
+            byte[] y = nh32(count);
             Y.write(y, 0, 8);
          }
 
          byte[] A = Y.toByteArray();
+         Y.reset();
          byte[] B;
          if (totalCount <= UMac32.L1_KEY_LEN) {
             // we might have 'update'd the bytes already. check
@@ -449,7 +448,6 @@ public class UHash32 extends BaseMac {
          }
 
          byte[] result = l3hash.digest(B);
-
          reset();
          return result;
       }
@@ -581,12 +579,9 @@ public class UHash32 extends BaseMac {
          this.highBound = that.highBound;
          this.bytesSoFar = that.bytesSoFar;
          if (that.buffer != null) {
+            byte[] thatbuffer = that.buffer.toByteArray();
             this.buffer = new ByteArrayOutputStream();
-            try {
-               that.buffer.writeTo(this.buffer);
-            } catch (IOException x) {
-               throw new ExceptionInInitializerError(x);
-            }
+            this.buffer.write(thatbuffer, 0, thatbuffer.length);
          }
       }
 
@@ -685,8 +680,8 @@ public class UHash32 extends BaseMac {
        * @param M string with length divisible by (wordbits / 8) bytes.
        * return y, integer in the range 0 .. prime(wordbits) - 1.
        */
-      private void
-      poly(int wordbits, BigInteger maxwordrange, BigInteger k, byte[] M, int off, int len) {
+      private void poly(int wordbits, BigInteger maxwordrange, BigInteger k,
+                        byte[] M, int off, int len) {
          byte[] mag = new byte[len];
          System.arraycopy(M, off, mag, 0, len);
          // Define constants used for fixing out-of-range words
@@ -758,8 +753,9 @@ public class UHash32 extends BaseMac {
          }
 
          // Break K1 into 8 chunks and convert to integers
-         int i = 0;
-         for (int j = 0; i < 8; ) {
+//         int i = 0;
+//         for (int j = 0; i < 8; ) {
+         for (int i = 0, j = 0; i < 8; i++) {
             long kk = (K1[j++] & 0xFFL) << 56 |
                       (K1[j++] & 0xFFL) << 48 |
                       (K1[j++] & 0xFFL) << 40 |
@@ -768,13 +764,11 @@ public class UHash32 extends BaseMac {
                       (K1[j++] & 0xFFL) << 16 |
                       (K1[j++] & 0xFFL) <<  8 |
                       (K1[j++] & 0xFFL);
-            k[i++] = (int)(kk % PRIME_36);
+//            k[i++] = (int)(kk % PRIME_36);
+            k[i] = (int)(kk % PRIME_36);
          }
-
-         k[i] = K2[0]         << 24 |
-               (K2[1] & 0xFF) << 16 |
-               (K2[2] & 0xFF) <<  8 |
-               (K2[3] & 0xFF);
+//         k[i] = K2[0] << 24 | (K2[1] & 0xFF) << 16 | (K2[2] & 0xFF) << 8 | (K2[3] & 0xFF);
+         k[8] = K2[0] << 24 | (K2[1] & 0xFF) << 16 | (K2[2] & 0xFF) << 8 | (K2[3] & 0xFF);
       }
 
       private L3Hash32(int[] k) {
@@ -792,7 +786,7 @@ public class UHash32 extends BaseMac {
       // java.lang.Cloneable interface implementation -------------------------
 
       public Object clone() {
-         return new L3Hash32(k);
+         return new L3Hash32((int[]) k.clone());
       }
 
       // other instance methods -----------------------------------------------
@@ -806,22 +800,18 @@ public class UHash32 extends BaseMac {
             throw new IllegalArgumentException("M length is not 16");
          }
 
-         long y = 0L;
+         long m, y = 0L;
          for (int i = 0, j = 0; i < 8; i++) {
             // Break M into 8 chunks and convert to integers
-            long m = (M[j++] & 0xFFL) << 8 | (M[j++] & 0xFFL);
+            m = (M[j++] & 0xFFL) << 8 | (M[j++] & 0xFFL);
 
             // Inner-product hash, extract last 32 bits and affine-translate
 //            y = (m_1 * k_1 + ... + m_8 * k_8) mod prime(36);
 //            y = y mod 2^32;
             y += (m * (k[i] & 0xFFFFFFFFL)) % PRIME_36;
          }
-
-         int Y = (int) y;
-         Y ^= k[8];
-         return new byte[] {
-            (byte)(Y >>> 24), (byte)(Y >>> 16), (byte)(Y >>> 8), (byte) Y
-         };
+         int Y = ((int) y) ^ k[8];
+         return new byte[] {(byte)(Y >>> 24), (byte)(Y >>> 16), (byte)(Y >>> 8), (byte) Y};
       }
    }
 }
