@@ -1,9 +1,9 @@
 package gnu.crypto.prng;
 
 // ----------------------------------------------------------------------------
-// $Id: PRNGFactory.java,v 1.7 2002/12/12 05:29:03 rsdio Exp $
+// $Id: PRNGFactory.java,v 1.9 2003/09/28 04:52:07 raif Exp $
 //
-// Copyright (C) 2001, 2002, Free Software Foundation, Inc.
+// Copyright (C) 2001, 2002, 2003  Free Software Foundation, Inc.
 //
 // This file is part of GNU Crypto.
 //
@@ -45,14 +45,19 @@ package gnu.crypto.prng;
 
 import gnu.crypto.Registry;
 
+import gnu.crypto.mac.IMac;
+import gnu.crypto.mac.MacFactory;
+import gnu.crypto.mac.HMacFactory;
+
 import java.util.Collections;
 import java.util.HashSet;
+import java.util.Iterator;
 import java.util.Set;
 
 /**
  * <p>A Factory to instantiate pseudo random number generators.</p>
  *
- * @version $Revision: 1.7 $
+ * @version $Revision: 1.9 $
  */
 public class PRNGFactory implements Registry {
 
@@ -94,6 +99,13 @@ public class PRNGFactory implements Registry {
          result = new MDGenerator();
       } else if (prng.equalsIgnoreCase(UMAC_PRNG)) {
          result = new UMacGenerator();
+      } else if (prng.toLowerCase().startsWith(PBKDF2_PRNG_PREFIX)) {
+         String macName = prng.substring(PBKDF2_PRNG_PREFIX.length());
+         IMac mac = MacFactory.getInstance(macName);
+         if (mac == null) {
+            return null;
+         }
+         result = new PBKDF2(mac);
       }
 
       return result;
@@ -111,6 +123,10 @@ public class PRNGFactory implements Registry {
       hs.add(ICM_PRNG);
       hs.add(MD_PRNG);
       hs.add(UMAC_PRNG);
+      // add all hmac implementations as candidate PBKDF2 ones too
+      for (Iterator it = HMacFactory.getNames().iterator(); it.hasNext(); ) {
+         hs.add(PBKDF2_PRNG_PREFIX+((String) it.next()));
+      }
 
       return Collections.unmodifiableSet(hs);
    }
